package tech.sbdevelopment.vehiclesplusconverter.utils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Version implements Comparable<Version> {
    private static final Pattern VERSION_PATTERN = Pattern.compile(
            "([0-9*]+)\\.([0-9*]+)(?:\\.([0-9*]+))?(?:\\.([0-9*]+))?(?:-SNAPSHOT)?"
    );

    private final Integer major;
    private final Integer minor;
    private final Integer patch;
    private final Integer build;

    public static Version of(String version) {
        return new Version(version);
    }

    Version(String version) {
        Matcher matcher = VERSION_PATTERN.matcher(version);
        if (matcher.matches()) {
            this.major = parsePart(matcher.group(1));
            this.minor = parsePart(matcher.group(2));
            this.patch = matcher.group(3) != null ? parsePart(matcher.group(3)) : null;
            this.build = matcher.group(4) != null ? parsePart(matcher.group(4)) : null;
        } else {
            throw new IllegalArgumentException("Invalid version format: " + version);
        }
    }

    private Integer parsePart(String part) {
        if ("*".equals(part)) {
            return null; // wildcard
        } else {
            return Integer.parseInt(part);
        }
    }

    public boolean isNewerThan(Version other) {
        return compareTo(other) > 0;
    }

    public boolean isOlderThan(Version other) {
        return compareTo(other) < 0;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) return true;
        if (obj == null || getClass() != obj.getClass()) return false;

        Version version = (Version) obj;

        return this.compareTo(version) == 0;
    }

    @Override
    public int compareTo(Version other) {
        int result = comparePart(this.major, other.major);
        if (result != 0) return result;

        result = comparePart(this.minor, other.minor);
        if (result != 0) return result;

        result = comparePart(this.patch, other.patch);
        if (result != 0) return result;

        return comparePart(this.build, other.build);
    }

    private int comparePart(Integer part1, Integer part2) {
        if (part1 == null && part2 == null) return 0;
        if (part1 == null) return -1; // wildcard is less than any number
        if (part2 == null) return 1;  // any number is greater than wildcard

        return Integer.compare(part1, part2);
    }

    @Override
    public int hashCode() {
        int result = (major != null ? major.hashCode() : 0);
        result = 31 * result + (minor != null ? minor.hashCode() : 0);
        result = 31 * result + (patch != null ? patch.hashCode() : 0);
        result = 31 * result + (build != null ? build.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(major != null ? major : "*").append('.')
                .append(minor != null ? minor : "*");
        if (patch != null || build != null) sb.append('.').append(patch != null ? patch : "*");
        if (build != null) sb.append('.').append(build);
        return sb.toString();
    }
}
